/* Copyright 2009 Fog Creek Software, Inc. */

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;

/* FogBugz namespaces-- make sure you add the neccesary assembly references to
 * the following DLL files contained in C:\Program Files\FogBugz\Website\bin\ 
 * FogBugz.dll, FogCreek.Plugins.dll, FogCreek.Plugins.InterfaceEvents.dll     */
using FogCreek.FogBugz.Plugins;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz;
using FogCreek.FogBugz.UI;

namespace IPluginStaticFile_Example
{
    /* Class Declaration: Inherit from Plugin, expose IPluginStaticFile, etc */
    public class IPluginStaticFile_Example : Plugin, IPluginStaticFile,
                                              IPluginPageDisplay, IPluginExtrasMenu
    {
        /* Dictionary of key value pairs for the permissions of our files */
        private Dictionary<string, PermissionLevel> rgFiles =
            new Dictionary<string, PermissionLevel>();
        /* pretty icons for each file type */
        private Dictionary<string, string> rgFileIcons =
            new Dictionary<string, string>();

        /* Constructor: We'll just initialize the inherited Plugin class, which 
         * takes the passed instance of CPluginApi and sets its "api" member variable.
         * we also need to populate our dictionaries of filenames */
        public IPluginStaticFile_Example(CPluginApi api): base(api)
        {
            /* populate the dictionary of files with their permission levels
             * The first two files are viewable by everyone, including non-users
             * so we can link to them somewhere outside of FogBugz */
            rgFiles.Add("EULA.txt", PermissionLevel.Public);
            rgFiles.Add("kiwi25.gif", PermissionLevel.Public);
            rgFiles.Add("admin_list.xls", PermissionLevel.Administrator);

            /* pretty icons for each file type */
            rgFileIcons.Add("txt", "txt.png");
            rgFileIcons.Add("gif", "img.png");
            rgFileIcons.Add("xls", "xls.png");
            rgFileIcons.Add("unk", "unk.png");
        }

        protected string HtmlFileList()
        {
            string sHtml = "";
            /* iterate over the list of files, determine the file type by extension,
             * display a link with an icon for the file type and show the permission */
            foreach (KeyValuePair<string, PermissionLevel> kvp in rgFiles)
            {
                string sExtension = kvp.Key.Substring(kvp.Key.Length - 3);
                string sIconFilename;
                try
                {
                    rgFileIcons.TryGetValue(sExtension, out sIconFilename);
                }
                catch (System.ArgumentNullException)
                {
                    sIconFilename = "unk.png";
                }
                sHtml += string.Format(@"<a href=""{0}"">
                                            <img style=""border: none"" src=""{1}"">
                                            {2}
                                         </a> ({3})<br />",
                                       api.Url.PluginStaticFileUrl(kvp.Key),
                                       api.Url.PluginStaticFileUrl(sIconFilename),
                                       kvp.Key,
                                       kvp.Value.ToString()
                );
            }
            return sHtml;
        }

        #region IPluginStaticFile Members

        public PermissionLevel StaticFileVisibility(string sFilename)
        {
            /* if the file is one of our icons, it should be visible to all */
            if (rgFileIcons.ContainsValue(sFilename))
            {
                return PermissionLevel.Public;
            }
            /* if the file is in the list, use the specified visibility value */
            else if (rgFiles.ContainsKey(sFilename))
            {
                PermissionLevel visibility;
                rgFiles.TryGetValue(sFilename, out visibility);
                return visibility;
            }
            /* anything else is a bogus file */
            else
            {
                throw new Exception(string.Format(@"File ""{0}"" does not exist",
                                                  sFilename));
            }
        }

        public void StaticFileHeaders(string sFilename)
        {
        }

        #endregion

        #region IPluginPageDisplay Members

        public string PageDisplay()
        {
            return string.Format(@"{0}
                                   <p>These are the company files available.
                                   The permissions required to download each
                                   are listed.</p>
                                   <p>{1}</p>",
                                   FogCreek.FogBugz.UI.PageDisplay.Headline(
                                        "Company Files"),
                                   HtmlFileList()
            );
        }

        public PermissionLevel PageVisibility()
        {
            /* only logged-in normal or administrator users can view the
             * Company Files page */
            return PermissionLevel.Normal;
        }

        #endregion

        #region IPluginExtrasMenu Members

        public CNavMenuLink[] ExtrasMenuLinks()
        {
            /* because the page we link to is only visible to normal and
             * administrator users, we should only add an Extras menu link for
             * those users */
            if (api.Person.GetCurrentPerson().GetPermissionLevel() >
                PermissionLevel.Community)
                return new CNavMenuLink[] { new CNavMenuLink("Company Files",
                                                             api.Url.PluginPageUrl())
                };
            else
                return new CNavMenuLink[] { };
        }

        #endregion
    }
}

